{% load i18n %}
<!doctype html>
<html lang="en">
<head>
  <meta charset="utf-8">
  <title>{% trans "401 Unauthorised" %}</title>
</head>
<body>
<h1>{% trans "401 Unauthorised" %}</h1>
<p>{{ reason }}</p>
</body>
</html>